-- Copyright (C) 1999 Daniel Elphick and others
-- Licensed under Eiffel Forum Freeware License, version 1;
-- (see forum.txt)
--
indexing

	description:	"This class is responsible for generating the eiffel code"
	author:			"Daniel Elphick <dre00r@ecs.soton.ac.uk>"
	cvs: "$Id: code_generator.e,v 1.20 2003/01/01 21:52:32 elphick Exp $"

class CODE_GENERATOR

inherit

	GLOBALS

	BASIC_DIRECTORY

	CLASS_INDEX

creation
	make

feature
	
	program_name: STRING
	directory: STRING
	src_directory: STRING
	interface: TAG_TREE

	make(tree: TAG_TREE; dirname: STRING) is
			-- Writes all the source files into the specified
			-- directory (if empty, current directory)
		require
			valid_tree: tree /= Void
		local
			temp_tree: TAG_TREE
			project: TAG_TREE
		do
			interface := tree.find_child("GTK-Interface")
			if interface = Void then
				print("GTK-Interface tag must be at root%N")
				die_with_code(exit_failure_code)
			end
			project := interface.find_child("project")
			if project = Void then
				print("GTK-Interface must have a project tag%N")
				die_with_code(exit_failure_code)
			end
			set_project_name(project.get_string_from_field("name"))
			program_name := project.get_string_from_field("program_name")
			-- Code to find the directory
			-- however I think this is useless as I've never seen glade fill
			-- this field in!
			if program_name.count = 0 then
				program_name.make_from_string(project_name)
			end
			temp_tree := project.find_child("directory")
			if temp_tree = Void then
				print("Project must have a directory field%N")
				die_with_code(exit_failure_code)
			end
			directory ?= temp_tree @ 0
			if directory.is_empty then
				directory := clone(dirname)
			end
			if directory.is_empty then
				directory := "."
			end
			temp_tree := project.find_child("source_directory")
			if temp_tree = Void then
				print("Project must have a source directory field%N")
				die_with_code(exit_failure_code)
			end
			src_directory ?= temp_tree @ 0
			if src_directory.is_empty then
				src_directory := directory
			else
				src_directory.precede('/')
				src_directory.prepend(directory)
			end
			generate_root_name(program_name, src_directory)
			print("Project name : " + project_name + "%N")
			print("Program name : " + program_name + "%N")
			print("Directory : " + directory +"%N")
			print("Source directory : " + src_directory + "%N")
		--	file_tools.mkdir(directory)
		--	file_tools.mkdir(src_directory)
			if basic_directory_open(directory.to_external).is_null then
				print("Could not open directory " + directory + "%N")
				die_with_code(exit_failure_code)
			end
			if basic_directory_open(src_directory.to_external).is_null  then
				if not basic_directory_mkdir(src_directory.to_external) then
					print("Could not create directory " + src_directory + "%N")
					die_with_code(exit_failure_code)
				end
			end
			src_directory.extend('/')
			make_classes (project)-- make all class
			write_loadpath
		end

	make_classes(project : TAG_TREE ) is
			-- add by Didier CLERC for legibility (better cutting):
			-- create all classes
		local
			root_class: ROOT_CLASS_WRITER
			top_level_class: TOP_LEVEL_CLASS_WRITER
			custom_class: CUSTOM_CLASS_WRITER
			version_class: VERSION_CLASS_WRITER
		do
			create root_class.make(interface, project, src_directory)
			root_class.write_gtk_classes
			root_class.write_class
			
			-- Creation of TOP_LEVEL_WINDOWS
			create top_level_class.make(interface, root_class.root_features,
										  src_directory)
			top_level_class.write_class
			
			-- Creation of CUSTOMISATION and VERSION   
			create custom_class.make(src_directory)
			custom_class.write_class
			create version_class.make(src_directory) 
			version_class.write_class
		end
	
	write_loadpath is
			-- Write loadpath.se which includes paths to eGTK
		local
			loadpath_file: TEXT_FILE_WRITE
			loadpath_file_name: STRING
		do
			loadpath_file_name := clone(src_directory)
			loadpath_file_name.append("loadpath.se")
			if file_exists(loadpath_file_name) then
				print("loadpath.se exists%N")
			else
			   !!loadpath_file.connect_to(loadpath_file_name)
			   if is_vegtk then
			      loadpath_file.put_string("${VEGTK}/VEGTK/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/enums/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/externals/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/externals/se/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/se/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/support/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/support/se/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/support/vese/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gtk/vese/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gdk/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gdk/enums/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gdk/enums/se/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gdk/externals/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/gdk/vese/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/support/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/support/se/%N")
			      loadpath_file.put_string("${VEGTK}/VEGTK/support/vese/%N")
			   else
			      loadpath_file.put_string("${EGNOME}/loadpath.se%N")
			      loadpath_file.put_string("${EGTK}/loadpath.se%N")
			   end
				loadpath_file.disconnect
				print("Wrote loadpath.se%N")
			end
		end
end
